/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <QObject>
#include <QWidget>
#include <QLineEdit>
#include <QPainter>
#include <QKeyEvent>
#include <QPaintEvent>
#include <QMouseEvent>
#include <QTimer>

#include "lineeditwc_widget.h"

LineEditWC::LineEditWC(QWidget *p) 
	: QLineEdit(p), chk_state(0), init_state(true) 
{ 
	setContextMenuPolicy(Qt::NoContextMenu); 
	timer = new QTimer(this);
	timer->setSingleShot(true);
	timer->setInterval(2000);
	connect(timer, SIGNAL(timeout()), this, SLOT(chk_clear()));	
}

void 
LineEditWC::setInitState(bool s) 
{ 
	init_state = s; 
	timer->stop();
	chk_clear();
	clear();
	if (init_state) {
		setStyleSheet("QLineEdit#wordInput {color: grey;}");
		setText("Enter a word");
		setCursorPosition(0);
	} else {
		setStyleSheet("QLineEdit#wordInput{color: black;}");
	}
}

void 
LineEditWC::paintEvent(QPaintEvent *ev) 
{
	QLineEdit::paintEvent(ev);

	QPainter r(this);
	switch (chk_state) {
	case 0:
		r.drawPixmap(size().width()-22, 0, QPixmap());
		break;
	case 1:
		r.drawPixmap(size().width()-22, (size().height()-16)/2, 
		QPixmap(QString(":/pics/tick.png")));
		break;
	case -1:
		r.drawPixmap(size().width()-22, (size().height()-16)/2, 
		QPixmap(QString(":/pics/cross.png")));
		break;
	}
}

void 
LineEditWC::mousePressEvent(QMouseEvent *ev)
{
	if (init_state) {
		setInitState(false);
		ev->accept();
	} else {
		QLineEdit::mousePressEvent(ev);
	}
}

void 
LineEditWC::keyPressEvent(QKeyEvent *ev)
{
	int k = ev->key();
	if (init_state) {
		if ((k >= Qt::Key_A && k <= Qt::Key_Z)) {
			setInitState(false);
			QLineEdit::keyPressEvent(ev);
		}
		ev->accept();
	} else {
		QLineEdit::keyPressEvent(ev);
	}
}

void 
LineEditWC::focusInEvent(QFocusEvent *ev)
{
	QLineEdit::focusInEvent(ev);
	if (init_state)
		setCursorPosition(0);
}

void 
LineEditWC::chk_clear() 
{ 
	chk_state = 0; 
	setStyleSheet("QLineEdit#wordInput {padding-right: 0;}");
}

void 
LineEditWC::chk(bool v)
{
	chk_state = v ? 1 : -1;

	chk_state == 1 ?  
		setStyleSheet("QLineEdit { padding-right: 28; border-width: 1px;"
					  "border-style: inset; border-color: darkgreen; }") 
		:
		setStyleSheet("QLineEdit { padding-right: 28; border-width: 1px;"
					  "border-style: inset; border-color: darkred; }");

	timer->start();
}

