/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#ifndef _MAIN_WIDGET_H_
#define _MAIN_WIDGET_H_

#include "ui_main_widget.h"
#include "spellathon_cls.h"

using Spellathon::SpellathonCls;

class QMainWindow;
class QLabel;
class QSettings;
class QProgressBar;
class QProgressDialog;
class QCloseEvent;
class PuzzleLetrs;
class SettingsW;

class MainWidget : public QMainWindow , public Ui::MainWindow
{
	Q_OBJECT
public:
	MainWidget(QWidget *parent = 0);
	~MainWidget();
	void init();
private:
	void closeEvent(QCloseEvent *event);
	void resizeEvent(QResizeEvent *event);
public slots:
	void printPuzzle(QString);
	void fatalMsg	(QString);
	void printMsg(QString, QString);
	void disableHints();
	void noDict(QString);
	void spellathonReady();
	void statProgressOpen(int, int);
	void statProgressClose();
	void statProgressUpdate(int);
	void progressDiagOpen(const QString&, const QString&, int, int);
	void progressDiagUpdate(int);
	void progressDiagClose();
private slots:
	void playPressed();
	void solvePressed();
	void finishPressed();
	void wordEntered();
	void showInst();
	void showAbout();
	void openSettings();
signals:
	void setWord(QString w);
	void play();
	void finish();
	void solve(QString s);
	void setSolRange(int, int);
	void setDoScore(bool);
	void setDoTime(bool);
	void wpl_update(int);
private:
	SpellathonCls	*spth;
	PuzzleLetrs	*puzzleLetrs_;
	SettingsW	*settingsW_;
	QLabel		*statWordCnt;
	QLabel		*statMsg;
	QDialog		*settingsDialog;
	QSettings	*settings;
	QString		pendingDictChange;
	QProgressBar *spbar;
	QProgressDialog *pbar;
};

#endif /* _MAIN_WIDGET_H_ */
