/*
 * Copyright (c) 2009, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * The names of its contributors may be used to endorse or promote products 
 * derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <QDialog>
#include <QShowEvent>
#include <QFont>
#include <QSettings>
#include <QFileDialog>
#include <QFontDialog>

#include "settings_widget.h"
#include "../logic/defines.h"

SettingsW::SettingsW(QWidget *p, QSettings *s) : QDialog(p), settings_(s) 
{ 	
	setupUi(this); 
	connect(this, SIGNAL(accepted()), SLOT(done()));
	connect(dictSelectButton, SIGNAL(clicked()), this, SLOT(changeDict()));
	connect(monoFontButton, SIGNAL(clicked()), this, SLOT(changeFontMono()));
	connect(spinBox_max, SIGNAL(valueChanged(int)), this, SLOT(adjustSbMin(int)));
	connect(spinBox_min, SIGNAL(valueChanged(int)), this, SLOT(adjustSbMax(int)));
	spinBox_max->setRange(MMIN_SOL + MGAP_SOL, MMAX_SOL);
	spinBox_min->setRange(MMIN_SOL, MMAX_SOL - MGAP_SOL);
}

void
SettingsW::showEvent(QShowEvent *ev)
{ 
	spinBox_min->setValue(settings_->value("solutions/min").toInt());
	spinBox_max->setValue(settings_->value("solutions/max").toInt());
	dictPathEdit->setText(settings_->value("dict/path").toString());
	precompCheckBox->setChecked(settings_->value("dict/precomp").toBool());
	scoreCheckBox->setChecked(settings_->value("score/show").toBool());
	timeCheckBox->setChecked(settings_->value("time/show").toBool());
	resize(settings_->value("settingswidget/size", size()).toSize());
	font = settings_->value("font/mono").value<QFont>();
	monoFontLineEdit->setFont(font);
	monoFontLineEdit->setText(QString("%1, %2pt").arg(font.family()).arg(font.pointSize()));
	ev->accept();
}

void
SettingsW::done()
{
	settings_->setValue("solutions/min", spinBox_min->value());
	settings_->setValue("solutions/max", spinBox_max->value());
	settings_->setValue("score/show", scoreCheckBox->isChecked());
	settings_->setValue("time/show", timeCheckBox->isChecked());
	settings_->setValue("font/mono", font);
	settings_->setValue("dict/path", dictPathEdit->text());
	settings_->setValue("dict/precomp", precompCheckBox->isChecked());
	settings_->setValue("settingswidget/size", size());
	settings_->sync();
}

void 
SettingsW::adjustSbMin(int v)
{
	if (v - spinBox_min->value() >= MGAP_SOL)
		spinBox_min->setMaximum(v - MGAP_SOL);
}

void 
SettingsW::adjustSbMax(int v)
{
	if (spinBox_max->value() - v >= MGAP_SOL)
		spinBox_max->setMinimum(v + MGAP_SOL);
}

void
SettingsW::changeDict()
{
	QString dpath = QFileDialog::getOpenFileName(this, "Choose Dictionary", 
												 settings_->value("dict/path").toString());

	if (dpath != "") 
		dictPathEdit->setText(dpath);
}

void
SettingsW::changeFontMono()
{
	bool ok;

	QFont f = QFontDialog::getFont(&ok, settings_->value("font/mono").value<QFont>(), this);
	if (ok) {
		font = f;
		monoFontLineEdit->setFont(f);
		monoFontLineEdit->setText(QString("%1, %2pt").arg(f.family()).arg(f.pointSize()));
	}
}


